#!/usr/bin/env python

import logging
import optparse
import os
import sys
from appdirs import AppDirs
from pkg_resources import resource_filename
from ansiblelater import classify
from ansiblelater.utils import info, warn, read_config, get_property


def main():
    config_dir = AppDirs("ansible-later").user_config_dir
    default_config_file = os.path.join(config_dir, "config.ini")

    parser = optparse.OptionParser("%prog playbook_file|role_file|inventory_file",
                                   version="%prog " + get_property("__version__"))
    parser.add_option('-c', dest='configfile', default=default_config_file,
                      help="Location of configuration file: [%s]" % default_config_file)
    parser.add_option('-d', dest='rulesdir',
                      help="Location of standards rules")
    parser.add_option('-q', dest='log_level', action="store_const", default=logging.WARN,
                      const=logging.ERROR, help="Only output errors")
    parser.add_option('-s', dest='standards_filter', action='append',
                      help="limit standards to specific names")
    parser.add_option('-v', dest='log_level', action="store_const", default=logging.WARN,
                      const=logging.INFO, help="Show more verbose output")

    options, args = parser.parse_args(sys.argv[1:])
    settings = read_config(options.configfile)

    # Merge CLI options with config options. CLI options override config options.
    for key, value in options.__dict__.items():
        if value:
            setattr(settings, key, value)

    if os.path.exists(settings.configfile):
        info("Using configuration file: %s" % settings.configfile, settings)
    else:
        warn("No configuration file found at %s" % settings.configfile, settings, file=sys.stderr)
        if not settings.rulesdir:
            rules_dir = os.path.join(resource_filename('ansiblelater', 'examples'))
            warn("Using example standards found at %s" % rules_dir, settings, file=sys.stderr)
            settings.rulesdir = rules_dir

    if len(args) == 0:
        candidates = []
        for root, dirs, files in os.walk("."):
            for filename in files:
                candidates.append(os.path.join(root, filename))
    else:
        candidates = args

    errors = 0
    for filename in candidates:
        if ':' in filename:
            (filename, lines) = filename.split(":")
        else:
            lines = None
        candidate = classify(filename)
        if candidate:
            if candidate.binary:
                info("Not reviewing binary file %s" % filename, settings)
                continue
            if candidate.vault:
                info("Not reviewing vault file %s" % filename, settings)
                continue
            if lines:
                info("Reviewing %s lines %s" % (candidate, lines), settings)
            else:
                info("Reviewing all of %s" % candidate, settings)
            errors = errors + candidate.review(settings, lines)
        else:
            info("Couldn't classify file %s" % filename, settings)
    return errors


main()



from __future__ import print_function

import importlib
import logging
import os
import subprocess
import sys
import re

from distutils.version import LooseVersion

try:
    import ConfigParser as configparser
except ImportError:
    import configparser

try:
    from ansible.utils.color import stringc
except ImportError:
    from ansible.color import stringc

# from yamlhelper import *


def abort(message, file=sys.stderr):
    print(stringc("FATAL: %s" % message, 'red'), file=file)
    sys.exit(1)


def error(message, file=sys.stderr):
    print(stringc("ERROR: %s" % message, 'red'), file=file)


def warn(message, settings, file=sys.stdout):
    if settings.log_level <= logging.WARNING:
        print(stringc("WARN: %s" % message, 'yellow'), file=file)


def info(message, settings, file=sys.stdout):
    if settings.log_level <= logging.INFO:
        print(stringc("INFO: %s" % message, 'green'), file=file)


def get_property(prop):
    currentdir = os.path.dirname(os.path.realpath(__file__))
    parentdir = os.path.dirname(currentdir)
    result = re.search(
        r'{}\s*=\s*[\'"]([^\'"]*)[\'"]'.format(prop),
        open(os.path.join(parentdir, '__init__.py')).read())
    return result.group(1)


def standards_latest(standards):
    return max([standard.version for standard in standards if standard.version] or ["0.1"],
               key=LooseVersion)


def lines_ranges(lines_spec):
    if not lines_spec:
        return None
    result = []
    for interval in lines_spec.split(","):
        (start, end) = interval.split("-")
        result.append(range(int(start), int(end) + 1))
    return result


def is_line_in_ranges(line, ranges):
    return not ranges or any([line in r for r in ranges])


def read_standards(settings):
    if not settings.rulesdir:
        abort("Standards directory is not set on command line or in configuration file - aborting")
    sys.path.append(os.path.abspath(os.path.expanduser(settings.rulesdir)))
    try:
        standards = importlib.import_module('standards')
    except ImportError as e:
        abort("Could not import standards from directory %s: %s" % (settings.rulesdir, str(e)))
    return standards


def read_config(config_file):
    config = configparser.RawConfigParser({'standards': None})
    config.read(config_file)

    return Settings(config, config_file)


def execute(cmd):
    result = ExecuteResult()
    encoding = 'UTF-8'
    env = dict(os.environ)
    env['PYTHONIOENCODING'] = encoding
    proc = subprocess.Popen(cmd, stdout=subprocess.PIPE,
                            stderr=subprocess.STDOUT, env=env)
    result.output = proc.communicate()[0].decode(encoding)
    result.rc = proc.returncode
    return result


class Settings(object):
    def __init__(self, config, config_file):
        self.rulesdir = None
        self.custom_modules = []
        self.log_level = None
        self.standards_filter = None

        if config.has_section('rules'):
            self.rulesdir = config.get('rules', 'standards')
        if config.has_section('ansible'):
            modules = config.get('ansible', 'custom_modules')
            self.custom_modules = [x.strip() for x in modules.split(',')]

        self.configfile = config_file


class ExecuteResult(object):
    pass



#!/usr/bin/env python
"""Setup script for the package."""

import re
import os
import io
from setuptools import find_packages
from setuptools import setup

PACKAGE_NAME = "ansiblelater"


def get_property(prop, project):
    current_dir = os.path.dirname(os.path.realpath(__file__))
    result = re.search(
        r'{}\s*=\s*[\'"]([^\'"]*)[\'"]'.format(prop),
        open(os.path.join(current_dir, project, '__init__.py')).read())
    return result.group(1)


def get_readme(filename='README.md'):
    this = os.path.abspath(os.path.dirname(__file__))
    with io.open(os.path.join(this, filename), encoding='utf-8') as f:
        long_description = f.read()
    return long_description


setup(
    name=get_property("__project__", PACKAGE_NAME),
    version=get_property("__version__", PACKAGE_NAME),
    description=("Reviews ansible playbooks, roles and inventories and suggests improvements."),
    keywords="ansible code review",
    author=get_property("__author__", PACKAGE_NAME),
    author_email=get_property("__email__", PACKAGE_NAME),
    url="https://github.com/xoxys/ansible-later",
    license=get_property("__license__", PACKAGE_NAME),
    long_description=get_readme(),
    long_description_content_type='text/markdown',
    packages=find_packages(exclude=["test", "test.*"]),
    python_requires='>=2.7,!=3.0.*,!=3.1.*,!=3.2.*,!=3.3.*,,!=3.4.*',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Console',
        'License :: OSI Approved :: MIT License',
        'Intended Audience :: Developers',
        'Intended Audience :: Information Technology',
        'Intended Audience :: System Administrators',
        'Natural Language :: English',
        'Operating System :: POSIX',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Topic :: Utilities',
    ],
    include_package_data=True,
    zip_safe=False,
    install_requires=[
        "ansible-lint>=3.4.1",
        "pyyaml",
        "appdirs",
        "unidiff",
        "flake8",
        "yamllint",
        "nested-lookup"
    ],
    entry_points={
        'console_scripts': [
            'ansible-later = ansiblelater.__main__:main'
        ]
    },
    test_suite="test"
)
